@@ -25,9 +25,9 @@
 
 typedef enum
 {
-  G_DNS_SD_DISPLAY_MODE_DISABLED,
-  G_DNS_SD_DISPLAY_MODE_SEPARATE,
-  G_DNS_SD_DISPLAY_MODE_MERGED
-} GDnsSdDisplayMode;
+  G_VFS_BACKEND_NETWORK_DISPLAY_MODE_DISABLED,
+  G_VFS_BACKEND_NETWORK_DISPLAY_MODE_SEPARATE,
+  G_VFS_BACKEND_NETWORK_DISPLAY_MODE_MERGED
+} GVfsBackendNetworkDisplayMode;
 
 #endif /* __gvfs_enums_h__ */
