@@ -108,27 +108,18 @@ copy_file_info_into (GFileInfo *src, GFileInfo *dest)
   g_strfreev (attrs);
 }
 
-typedef enum
-{
-  AFP_HANDLE_TYPE_READ_FILE,
-  AFP_HANDLE_TYPE_CREATE_FILE,
-  AFP_HANDLE_TYPE_REPLACE_FILE_TEMP,
-  AFP_HANDLE_TYPE_REPLACE_FILE_DIRECT,
-  AFP_HANDLE_TYPE_APPEND_TO_FILE
-} AfpHandleType;
-
 typedef struct
 {
   GVfsBackendAfp *backend;
-  
-  AfpHandleType type;
+
   gint16 fork_refnum;
   gint64 offset;
-  
-  /* Used if type == AFP_HANDLE_TYPE_REPLACE_FILE_DIRECT */
-  gint64 size;
 
-  /* Used if type == AFP_HANDLE_TYPE_REPLACE_FILE_TEMP */
+  /* For write only */
+  gint64 size;
+  GVfsJobOpenForWriteMode mode;
+
+  /* For replace only */
   char *filename;
   char *tmp_filename;
   gboolean make_backup;
@@ -692,9 +683,7 @@ write_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
 
   written_size = last_written - afp_handle->offset;
   afp_handle->offset = last_written;
-
-  if (afp_handle->type == AFP_HANDLE_TYPE_REPLACE_FILE_DIRECT)
-    afp_handle->size = MAX (last_written, afp_handle->size);
+  afp_handle->size = MAX (last_written, afp_handle->size);
   
   g_vfs_job_write_set_written_size (job, written_size); 
   g_vfs_job_succeeded (G_VFS_JOB (job));
@@ -710,6 +699,9 @@ try_write (GVfsBackend *backend,
   GVfsBackendAfp *afp_backend = G_VFS_BACKEND_AFP (backend);
   AfpHandle *afp_handle = (AfpHandle *)handle;
 
+  if (afp_handle->mode == OPEN_FOR_WRITE_APPEND)
+    afp_handle->offset = afp_handle->size;
+
   g_vfs_afp_volume_write_to_fork (afp_backend->volume, afp_handle->fork_refnum,
                                   buffer, buffer_size, afp_handle->offset,
                                   G_VFS_JOB (job)->cancellable, write_cb, job);
@@ -717,37 +709,6 @@ try_write (GVfsBackend *backend,
   return TRUE;
 }
 
-static void
-seek_on_write_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
-{
-  GVfsAfpVolume *volume = G_VFS_AFP_VOLUME (source_object);
-  GVfsJobSeekWrite *job = G_VFS_JOB_SEEK_WRITE (user_data);
-  AfpHandle *afp_handle = (AfpHandle *)job->handle;
-
-  GError *err = NULL;
-  GFileInfo *info;
-  gsize size;
-
-  info = g_vfs_afp_volume_get_fork_parms_finish (volume, res, &err);
-  if (!info)
-  {
-    g_vfs_job_failed_from_error (G_VFS_JOB (job), err);
-    g_error_free (err);
-    return;
-  }
-  
-  size = g_file_info_get_size (info);
-  g_object_unref (info);
-
-  afp_handle->offset = size + job->requested_offset;
-
-  if (afp_handle->offset < 0)
-    afp_handle->offset = 0;
-
-  g_vfs_job_seek_write_set_offset (job, afp_handle->offset);
-  g_vfs_job_succeeded (G_VFS_JOB (job));
-}
-
 static gboolean
 try_seek_on_write (GVfsBackend *backend,
                    GVfsJobSeekWrite *job,
@@ -755,39 +716,27 @@ try_seek_on_write (GVfsBackend *backend,
                    goffset    offset,
                    GSeekType  type)
 {
-  GVfsBackendAfp *afp_backend = G_VFS_BACKEND_AFP (backend);
   AfpHandle *afp_handle = (AfpHandle *)handle;
 
-  if (afp_handle->type == AFP_HANDLE_TYPE_REPLACE_FILE_DIRECT ||
-      job->seek_type != G_SEEK_END)
+  switch (job->seek_type)
   {
-    switch (job->seek_type)
-    {
-      case G_SEEK_CUR:
-        afp_handle->offset += job->requested_offset;
-        break;
-      case G_SEEK_SET:
-        afp_handle->offset = job->requested_offset;
-        break;
-      case G_SEEK_END:
-        afp_handle->offset = afp_handle->size + job->requested_offset;
-        break;
-    }
-
-    if (afp_handle->offset < 0)
-      afp_handle->offset = 0;
-
-    g_vfs_job_seek_write_set_offset (job, afp_handle->offset);
-    g_vfs_job_succeeded (G_VFS_JOB (job));
+    case G_SEEK_CUR:
+      afp_handle->offset += job->requested_offset;
+      break;
+    case G_SEEK_SET:
+      afp_handle->offset = job->requested_offset;
+      break;
+    case G_SEEK_END:
+      afp_handle->offset = afp_handle->size + job->requested_offset;
+      break;
   }
-  
-  else
-  {
-    g_vfs_afp_volume_get_fork_parms (afp_backend->volume, afp_handle->fork_refnum,
-                                     AFP_FILE_BITMAP_EXT_DATA_FORK_LEN_BIT,
-                                     G_VFS_JOB (job)->cancellable, seek_on_write_cb, job);
-  }
-    
+
+  if (afp_handle->offset < 0)
+    afp_handle->offset = 0;
+
+  g_vfs_job_seek_write_set_offset (job, afp_handle->offset);
+  g_vfs_job_succeeded (G_VFS_JOB (job));
+
   return TRUE;
 }
 
@@ -796,11 +745,13 @@ truncate_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
   GVfsAfpVolume *volume = G_VFS_AFP_VOLUME (source_object);
   GVfsJobTruncate *job = G_VFS_JOB_TRUNCATE (user_data);
+  AfpHandle *afp_handle = (AfpHandle *)job->handle;
 
   GError *err = NULL;
 
   if (g_vfs_afp_volume_set_fork_size_finish (volume, res, &err))
   {
+    afp_handle->size = job->size;
     g_vfs_job_succeeded (G_VFS_JOB (job));
   }
   else
@@ -1068,31 +1019,6 @@ close_write_get_fork_parms_cb (GObject *source_object, GAsyncResult *res, gpoint
 
   close_fork (volume, G_VFS_JOB (job), afp_handle);
 }
-  
-static void
-close_replace_set_fork_size_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
-{
-  GVfsAfpVolume *volume = G_VFS_AFP_VOLUME (source_object);
-  GVfsJobCloseWrite *job = G_VFS_JOB_CLOSE_WRITE (user_data);
-  GVfsBackendAfp *afp_backend = G_VFS_BACKEND_AFP (job->backend);
-  AfpHandle *afp_handle = (AfpHandle *)job->handle;
-
-  GError *err = NULL;
-
-  if (!g_vfs_afp_volume_set_fork_size_finish (volume, res, &err))
-  {
-    g_vfs_job_failed_from_error (G_VFS_JOB (job), err);
-    g_error_free (err);
-    afp_handle_free (afp_handle);
-    return;
-  }
-
-  /* Get ETAG */
-  g_vfs_afp_volume_get_fork_parms (afp_backend->volume, afp_handle->fork_refnum,
-                                   AFP_FILE_BITMAP_MOD_DATE_BIT,
-                                   G_VFS_JOB (job)->cancellable,
-                                   close_write_get_fork_parms_cb, job);
-}
 
 static gboolean
 try_close_write (GVfsBackend *backend,
@@ -1102,19 +1028,13 @@ try_close_write (GVfsBackend *backend,
   GVfsBackendAfp *afp_backend = G_VFS_BACKEND_AFP (backend);
   AfpHandle *afp_handle = (AfpHandle *)handle;
   
-  if (afp_handle->type == AFP_HANDLE_TYPE_REPLACE_FILE_TEMP)
+  if (afp_handle->tmp_filename != NULL)
   {
     g_vfs_afp_volume_exchange_files (afp_backend->volume, afp_handle->filename,
                                      afp_handle->tmp_filename, 
                                      G_VFS_JOB (job)->cancellable,
                                      close_replace_exchange_files_cb, job);
   }
-  else if (afp_handle->type == AFP_HANDLE_TYPE_REPLACE_FILE_DIRECT)
-  {
-    g_vfs_afp_volume_set_fork_size (afp_backend->volume, afp_handle->fork_refnum,
-                                    afp_handle->size, G_VFS_JOB (job)->cancellable,
-                                    close_replace_set_fork_size_cb, job);
-  }
   else
   {
     /* Get ETAG */
@@ -1141,7 +1061,40 @@ try_close_read (GVfsBackend *backend,
 }
 
 static void
-create_open_fork_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
+open_for_write_get_fork_parms_cb (GObject *source_object,
+                                  GAsyncResult *res,
+                                  gpointer user_data)
+{
+  GVfsAfpVolume *volume = G_VFS_AFP_VOLUME (source_object);
+  GVfsJobOpenForWrite *job = G_VFS_JOB_OPEN_FOR_WRITE (user_data);
+  AfpHandle *afp_handle = (AfpHandle *)job->backend_handle;
+  GFileInfo *info;
+  GError *err = NULL;
+  goffset size;
+
+  info = g_vfs_afp_volume_get_fork_parms_finish (volume, res, &err);
+  if (!info)
+  {
+    g_vfs_job_failed_from_error (G_VFS_JOB (job), err);
+    g_error_free (err);
+    afp_handle_free (afp_handle);
+    return;
+  }
+
+  size = g_file_info_get_size (info);
+  g_object_unref (info);
+
+  afp_handle->offset = size;
+  afp_handle->size = size;
+  g_vfs_job_open_for_write_set_initial_offset (job, size);
+
+  g_vfs_job_succeeded (G_VFS_JOB (job));
+}
+
+static void
+open_for_write_open_fork_cb (GObject *source_object,
+                             GAsyncResult *res,
+                             gpointer user_data)
 {
   GVfsAfpVolume *volume = G_VFS_AFP_VOLUME (source_object);
   GVfsJobOpenForWrite *job = G_VFS_JOB_OPEN_FOR_WRITE (user_data);
@@ -1159,33 +1112,68 @@ create_open_fork_cb (GObject *source_object, GAsyncResult *res, gpointer user_da
   }
 
   afp_handle = afp_handle_new (afp_backend, fork_refnum);
-  afp_handle->type = AFP_HANDLE_TYPE_CREATE_FILE;
+  afp_handle->mode = job->mode;
   
   g_vfs_job_open_for_write_set_handle (job, (GVfsBackendHandle) afp_handle);
   g_vfs_job_open_for_write_set_can_seek (job, TRUE);
   g_vfs_job_open_for_write_set_can_truncate (job, TRUE);
-  g_vfs_job_open_for_write_set_initial_offset (job, 0);
+
+  if (job->mode == OPEN_FOR_WRITE_APPEND)
+    {
+      g_vfs_afp_volume_get_fork_parms (afp_backend->volume,
+                                       afp_handle->fork_refnum,
+                                       AFP_FILE_BITMAP_EXT_DATA_FORK_LEN_BIT,
+                                       G_VFS_JOB (job)->cancellable,
+                                       open_for_write_get_fork_parms_cb,
+                                       job);
+      return ;
+    }
 
   g_vfs_job_succeeded (G_VFS_JOB (job));
 }
 
 static void
-create_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
+open_for_write_create_file_cb (GObject *source_object,
+                               GAsyncResult *res,
+                               gpointer user_data)
 {
   GVfsAfpVolume *volume = G_VFS_AFP_VOLUME (source_object);
   GVfsJobOpenForWrite *job = G_VFS_JOB_OPEN_FOR_WRITE (user_data);
 
   GError *err = NULL;
 
-  if (!g_vfs_afp_volume_create_file_finish (volume, res, &err))
+  if (!g_vfs_afp_volume_create_file_finish (volume, res, &err) &&
+      (job->mode == OPEN_FOR_WRITE_CREATE ||
+       !g_error_matches (err, G_IO_ERROR, G_IO_ERROR_EXISTS)))
   {
     g_vfs_job_failed_from_error (G_VFS_JOB (job), err);
     g_error_free (err);
     return;
   }
 
-  g_vfs_afp_volume_open_fork (volume, job->filename, AFP_ACCESS_MODE_WRITE_BIT, 0,
-                              G_VFS_JOB (job)->cancellable, create_open_fork_cb, job);
+  g_clear_error (&err);
+  g_vfs_afp_volume_open_fork (volume,
+                              job->filename,
+                              AFP_ACCESS_MODE_WRITE_BIT,
+                              0,
+                              G_VFS_JOB (job)->cancellable,
+                              open_for_write_open_fork_cb,
+                              job);
+}
+
+static void
+open_for_write (GVfsBackend *backend,
+                GVfsJobOpenForWrite *job,
+                const char *filename)
+{
+  GVfsBackendAfp *afp_backend = G_VFS_BACKEND_AFP (backend);
+
+  g_vfs_afp_volume_create_file (afp_backend->volume,
+                                filename,
+                                FALSE,
+                                G_VFS_JOB (job)->cancellable,
+                                open_for_write_create_file_cb,
+                                job);
 }
 
 static gboolean
@@ -1194,14 +1182,37 @@ try_create (GVfsBackend *backend,
             const char *filename,
             GFileCreateFlags flags)
 {
-  GVfsBackendAfp *afp_backend = G_VFS_BACKEND_AFP (backend);
-
-  g_vfs_afp_volume_create_file (afp_backend->volume, filename, FALSE,
-                                G_VFS_JOB (job)->cancellable, create_cb, job);
+  open_for_write (backend, job, filename);
 
   return TRUE;
 }
 
+static void
+replace_set_fork_size_cb (GObject *source_object,
+                          GAsyncResult *res,
+                          gpointer user_data)
+{
+  GVfsAfpVolume *volume = G_VFS_AFP_VOLUME (source_object);
+  GVfsJobTruncate *job = G_VFS_JOB_TRUNCATE (user_data);
+  AfpHandle *afp_handle = (AfpHandle *)job->handle;
+  GError *err = NULL;
+
+  if (!g_vfs_afp_volume_set_fork_size_finish (volume, res, &err))
+  {
+    g_vfs_afp_volume_close_fork (volume,
+                                 afp_handle->fork_refnum,
+                                 G_VFS_JOB (job)->cancellable,
+                                 NULL,
+                                 NULL);
+    afp_handle_free (afp_handle);
+    g_vfs_job_failed_from_error (G_VFS_JOB (job), err);
+    g_error_free (err);
+    return;
+  }
+
+  g_vfs_job_succeeded (G_VFS_JOB (job));
+}
+
 static void
 replace_open_fork_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
@@ -1222,22 +1233,31 @@ replace_open_fork_cb (GObject *source_object, GAsyncResult *res, gpointer user_d
   }
 
   afp_handle = afp_handle_new (afp_backend, fork_refnum);
+  afp_handle->mode = job->mode;
+
+  g_vfs_job_open_for_write_set_handle (job, (GVfsBackendHandle) afp_handle);
+  g_vfs_job_open_for_write_set_can_seek (job, TRUE);
+  g_vfs_job_open_for_write_set_can_truncate (job, TRUE);
+  g_vfs_job_open_for_write_set_initial_offset (job, 0);
+
   tmp_filename = g_object_get_data (G_OBJECT (job), "TempFilename");
   /* Replace using temporary file */
   if (tmp_filename)
   {
-    afp_handle->type = AFP_HANDLE_TYPE_REPLACE_FILE_TEMP;
     afp_handle->filename = g_strdup (job->filename);
     afp_handle->tmp_filename = g_strdup (tmp_filename);
     afp_handle->make_backup = job->make_backup;
   }
   else
-    afp_handle->type = AFP_HANDLE_TYPE_REPLACE_FILE_DIRECT;
-    
-  g_vfs_job_open_for_write_set_handle (job, (GVfsBackendHandle) afp_handle);
-  g_vfs_job_open_for_write_set_can_seek (job, TRUE);
-  g_vfs_job_open_for_write_set_can_truncate (job, TRUE);
-  g_vfs_job_open_for_write_set_initial_offset (job, 0);
+  {
+    g_vfs_afp_volume_set_fork_size (volume,
+                                    fork_refnum,
+                                    0,
+                                    G_VFS_JOB (job)->cancellable,
+                                    replace_set_fork_size_cb,
+                                    job);
+    return;
+  }
 
   g_vfs_job_succeeded (G_VFS_JOB (job));
 }
@@ -1389,82 +1409,25 @@ try_replace (GVfsBackend *backend,
   return TRUE;
 }
 
-static void
-append_to_get_fork_parms_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
-{
-  GVfsAfpVolume *volume = G_VFS_AFP_VOLUME (source_object);
-  GVfsJobOpenForWrite *job = G_VFS_JOB_OPEN_FOR_WRITE (user_data);
-  AfpHandle *afp_handle = (AfpHandle *)job->backend_handle;
-
-  GFileInfo *info;
-  GError *err = NULL;
-  goffset size;
-
-  info = g_vfs_afp_volume_get_fork_parms_finish (volume, res, &err);
-  if (!info)
-  {
-    g_vfs_job_failed_from_error (G_VFS_JOB (job), err); 
-    g_error_free (err);
-    afp_handle_free (afp_handle);
-    return;
-  }
-
-  size = g_file_info_get_size (info);
-  g_object_unref (info);
-
-  afp_handle->offset = size;
-  g_vfs_job_open_for_write_set_initial_offset (job, size);
-  g_vfs_job_open_for_write_set_can_seek (job, TRUE);
-  g_vfs_job_open_for_write_set_can_truncate (job, TRUE);
-
-  g_vfs_job_succeeded (G_VFS_JOB (job));
-}
-
-static void
-append_to_open_fork_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
-{
-  GVfsAfpVolume *volume = G_VFS_AFP_VOLUME (source_object);
-  GVfsJobOpenForWrite *job = G_VFS_JOB_OPEN_FOR_WRITE (user_data);
-  GVfsBackendAfp *afp_backend = G_VFS_BACKEND_AFP (job->backend);
-
-  gint16 fork_refnum;
-  GError *err = NULL;
-  AfpHandle *afp_handle;
-
-  if (!g_vfs_afp_volume_open_fork_finish (volume, res, &fork_refnum, NULL, &err))
-  {
-    /* Create file if it doesn't exist */
-    if (g_error_matches (err, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
-      try_create (G_VFS_BACKEND (afp_backend), job, job->filename, job->flags);
-    
-    else
-      g_vfs_job_failed_from_error (G_VFS_JOB (job), err);
-    
-    g_error_free (err);
-    return;
-  }
-
-  afp_handle = afp_handle_new (afp_backend, fork_refnum);
-  afp_handle->type = AFP_HANDLE_TYPE_APPEND_TO_FILE;
-  g_vfs_job_open_for_write_set_handle (job, (GVfsBackendHandle) afp_handle);
-  
-  g_vfs_afp_volume_get_fork_parms (afp_backend->volume, afp_handle->fork_refnum,
-                                   AFP_FILE_BITMAP_EXT_DATA_FORK_LEN_BIT,
-                                   G_VFS_JOB (job)->cancellable,
-                                   append_to_get_fork_parms_cb, job);
-}
-
 static gboolean
 try_append_to (GVfsBackend *backend,
                GVfsJobOpenForWrite *job,
                const char *filename,
                GFileCreateFlags flags)
 {
-  GVfsBackendAfp *afp_backend = G_VFS_BACKEND_AFP (backend);
+  open_for_write (backend, job, filename);
+
+  return TRUE;
+}
+
+static gboolean
+try_edit (GVfsBackend *backend,
+          GVfsJobOpenForWrite *job,
+          const char *filename,
+          GFileCreateFlags flags)
+{
+  open_for_write (backend, job, filename);
 
-  g_vfs_afp_volume_open_fork (afp_backend->volume, job->filename,
-                              AFP_ACCESS_MODE_WRITE_BIT, 0,
-                              G_VFS_JOB (job)->cancellable, append_to_open_fork_cb, job);
   return TRUE;
 }
 
@@ -1487,7 +1450,6 @@ read_open_fork_cb (GObject *source_object, GAsyncResult *res, gpointer user_data
   }
 
   afp_handle = afp_handle_new (afp_backend, fork_refnum);
-  afp_handle->type = AFP_HANDLE_TYPE_READ_FILE;
   
   g_vfs_job_open_for_read_set_handle (job, (GVfsBackendHandle) afp_handle);
   g_vfs_job_open_for_read_set_can_seek (job, TRUE);
@@ -2216,6 +2178,7 @@ g_vfs_backend_afp_class_init (GVfsBackendAfpClass *klass)
   backend_class->try_read = try_read;
   backend_class->try_seek_on_read = try_seek_on_read;
   backend_class->try_append_to = try_append_to;
+  backend_class->try_edit = try_edit;
   backend_class->try_create = try_create;
   backend_class->try_replace = try_replace;
   backend_class->try_write = try_write;
