@@ -47,7 +47,8 @@ typedef enum {
   G_VFS_FTP_FEATURE_AUTH_TLS,
   G_VFS_FTP_FEATURE_AUTH_SSL,
   G_VFS_FTP_FEATURE_CHMOD,
-  G_VFS_FTP_FEATURE_CHGRP
+  G_VFS_FTP_FEATURE_CHGRP,
+  G_VFS_FTP_FEATURE_MFMT
 } GVfsFtpFeature;
 #define G_VFS_FTP_FEATURES_DEFAULT (0)
 
