@@ -739,7 +739,7 @@ handle_register_mount (GVfsDBusMountTracker *object,
       if (arg_user_visible)
         {
           /* Use the old .gvfs location as fallback, not .cache/gvfs */
-          if (g_get_user_runtime_dir() == g_get_user_cache_dir ())
+          if (g_strcmp0 (g_get_user_runtime_dir(), g_get_user_cache_dir ()) == 0)
             mount->fuse_mountpoint = g_build_filename (g_get_home_dir(), ".gvfs", mount->stable_name, NULL);
           else
             mount->fuse_mountpoint = g_build_filename (g_get_user_runtime_dir(), "gvfs", mount->stable_name, NULL);
